const AWS = require('aws-sdk');

class AWSLambdaCaller {
  constructor(awsConfig, otherParams = {}) {
    this.config = awsConfig;
    this.otherParams = otherParams || {};
  }

  async call(jsonObj, functionName) {
    const lambda = new AWS.Lambda(Object.assign(this.config, this.otherParams));

    const params = {
      FunctionName: functionName,
      Payload: JSON.stringify(jsonObj),
    };

    try {
      return await lambda
        .invoke(params)
        .promise()
        .then(value => {
          return JSON.parse(value.Payload);
        });
    } catch (e) {
      console.warn('Error, cannot invoke lambda ' + functionName, e);
      throw e;
    }
  }
}

module.exports = AWSLambdaCaller;
